@@ -12,7 +12,8 @@ class LFUCache(Cache):
 
     def __getitem__(self, key, cache_getitem=Cache.__getitem__):
         value = cache_getitem(self, key)
-        self.__counter[key] -= 1
+        if key in self:  # __missing__ may not store item
+            self.__counter[key] -= 1
         return value
 
     def __setitem__(self, key, value, cache_setitem=Cache.__setitem__):
@@ -28,6 +29,6 @@ class LFUCache(Cache):
         try:
             (key, _), = self.__counter.most_common(1)
         except ValueError:
-            raise KeyError('%s is empty' % self.__class__.__name__)
+            raise KeyError('%s is empty' % type(self).__name__) from None
         else:
             return (key, self.pop(key))
