@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import collections
 
 from .cache import Cache
@@ -8,13 +6,14 @@ from .cache import Cache
 class LRUCache(Cache):
     """Least Recently Used (LRU) cache implementation."""
 
-    def __init__(self, maxsize, missing=None, getsizeof=None):
-        Cache.__init__(self, maxsize, missing, getsizeof)
+    def __init__(self, maxsize, getsizeof=None):
+        Cache.__init__(self, maxsize, getsizeof)
         self.__order = collections.OrderedDict()
 
     def __getitem__(self, key, cache_getitem=Cache.__getitem__):
         value = cache_getitem(self, key)
-        self.__update(key)
+        if key in self:  # __missing__ may not store item
+            self.__update(key)
         return value
 
     def __setitem__(self, key, value, cache_setitem=Cache.__setitem__):
@@ -30,19 +29,12 @@ class LRUCache(Cache):
         try:
             key = next(iter(self.__order))
         except StopIteration:
-            raise KeyError('%s is empty' % self.__class__.__name__)
+            raise KeyError('%s is empty' % type(self).__name__) from None
         else:
             return (key, self.pop(key))
 
-    if hasattr(collections.OrderedDict, 'move_to_end'):
-        def __update(self, key):
-            try:
-                self.__order.move_to_end(key)
-            except KeyError:
-                self.__order[key] = None
-    else:
-        def __update(self, key):
-            try:
-                self.__order[key] = self.__order.pop(key)
-            except KeyError:
-                self.__order[key] = None
+    def __update(self, key):
+        try:
+            self.__order.move_to_end(key)
+        except KeyError:
+            self.__order[key] = None
