@@ -3,13 +3,21 @@ import random
 from .cache import Cache
 
 
+# random.choice cannot be pickled in Python 2.7
+def _choice(seq):
+    return random.choice(seq)
+
+
 class RRCache(Cache):
     """Random Replacement (RR) cache implementation."""
 
-    def __init__(self, maxsize, choice=random.choice, missing=None,
-                 getsizeof=None):
-        Cache.__init__(self, maxsize, missing, getsizeof)
-        self.__choice = choice
+    def __init__(self, maxsize, choice=random.choice, getsizeof=None):
+        Cache.__init__(self, maxsize, getsizeof)
+        # TODO: use None as default, assing to self.choice directly?
+        if choice is random.choice:
+            self.__choice = _choice
+        else:
+            self.__choice = choice
 
     @property
     def choice(self):
@@ -21,6 +29,6 @@ class RRCache(Cache):
         try:
             key = self.__choice(list(self))
         except IndexError:
-            raise KeyError('%s is empty' % self.__class__.__name__)
+            raise KeyError('%s is empty' % type(self).__name__) from None
         else:
             return (key, self.pop(key))
