@@ -43,3 +43,16 @@ class CacheKeysTest(unittest.TestCase):
         self.assertIsInstance(key(1, 2, 3) + key(4, 5, 6), type(key()))
         self.assertIsInstance(key(1, 2, 3) + (4, 5, 6), type(key()))
         self.assertIsInstance((1, 2, 3) + key(4, 5, 6), type(key()))
+
+    def test_pickle(self, key=cachetools.keys.hashkey):
+        import pickle
+
+        for k in [key(), key('abc'), key('abc', 123), key('abc', q='abc')]:
+            # white-box test: assert cached hash value is not pickled
+            self.assertEqual(len(k.__dict__), 0)
+            h = hash(k)
+            self.assertEqual(len(k.__dict__), 1)
+            pickled = pickle.loads(pickle.dumps(k))
+            self.assertEqual(len(pickled.__dict__), 0)
+            self.assertEqual(k, pickled)
+            self.assertEqual(h, hash(pickled))
